scriptname GuardPlacementThread extends Quest hidden
 
;Thread variables
ObjectReference future
bool thread_queued = false
 
;Variables you need to get things done go here 
ActorBase theGuard
Static theMarker
 
;Thread queuing and set-up
ObjectReference function get_async(Activator akFuture, ObjectReference akFutureAnchor, ActorBase akGuard, Static akXMarker)
 
    ;Let the Thread Manager know that this thread is busy
    thread_queued = true
 
    ;Store our passed-in parameters to member variables
	theGuard = akGuard
	theMarker = akXMarker
 
	;Create the Future that will contain our result
	future = akFutureAnchor.PlaceAtMe(akFuture)
	return future
endFunction
 
;Allows the Thread Manager to determine if this thread is available
bool function busy()
	return thread_queued
endFunction
 
;For Thread Manager troubleshooting.
bool function has_future(ObjectReference akFuture)
    if akFuture == future
        return true
    else
        return false
    endif
endFunction
 
;For Thread Manager troubleshooting.
bool function force_unlock()
    clear_thread_vars()
    thread_queued = false
    return true
endFunction
 
;The actual set of code we want to multithread.
Event OnGuardPlacement()
	if thread_queued
		;OK, let's get some work done!
		ObjectReference tempMarker = Game.GetPlayer().PlaceAtMe(theMarker)
		MoveGuardMarkerNearPlayer(tempMarker)
		ObjectReference result = tempMarker.PlaceAtMe(theGuard)
		tempMarker.Disable()
		tempMarker.Delete()
 
        ;OK, we're done - let's pass the result back to the future
		(future as GuardPlacementFuture).result = result
 
        ;Set all variables back to default
		clear_thread_vars()
 
        ;Make the thread available to the Thread Manager again
		thread_queued = false
	endif
endEvent
 
;Called from Event OnGuardPlacement
function MoveGuardMarkerNearPlayer(ObjectReference akMarker)
	;Move the marker away from the player a random distance and direction in 75.0 game unit increments
	Actor player = Game.GetPlayer()
	Float A = player.GetAngleZ() + (Utility.RandomInt(1, 24) * 15.0)
	Float YDist = math.Sin(A)
	Float XDist = math.Cos(A)
	XDist *= (Utility.RandomInt(1, 5) * 75.0)
	YDist *= (Utility.RandomInt(1, 5) * 75.0)
	akMarker.MoveTo(player, XDist, YDist)
EndFunction
 
function clear_thread_vars()
	;Reset all thread variables to default state
	theGuard = None
	theMarker = None
endFunction